@@ -1,4 +1,4 @@
-from typing import Callable, Dict, List, Tuple
+from typing import Callable, Dict, List, Tuple, Union
 
 import torch
 import torch.nn as nn
@@ -30,16 +30,51 @@ class LIGAttributions(Attributions):
         input_ids: torch.Tensor,
         ref_input_ids: torch.Tensor,
         sep_id: int,
+        attention_mask: torch.Tensor,
+        token_type_ids: torch.Tensor = None,
+        position_ids: torch.Tensor = None,
+        ref_token_type_ids: torch.Tensor = None,
+        ref_position_ids: torch.Tensor = None,
     ):
         super().__init__(custom_forward, embeddings, tokens)
         self.input_ids = input_ids
         self.ref_input_ids = ref_input_ids
+        self.attention_mask = attention_mask
+        self.token_type_ids = token_type_ids
+        self.position_ids = position_ids
+        self.ref_token_type_ids = ref_token_type_ids
+        self.ref_position_ids = ref_position_ids
+
         self.lig = LayerIntegratedGradients(self.custom_forward, self.embeddings)
-        self._attributions, self.delta = self.lig.attribute(
-            inputs=self.input_ids,
-            baselines=self.ref_input_ids,
-            return_convergence_delta=True,
-        )
+
+        if self.token_type_ids is not None and self.position_ids is not None:
+            self._attributions, self.delta = self.lig.attribute(
+                inputs=(self.input_ids, self.token_type_ids, self.position_ids),
+                baselines=(
+                    self.ref_input_ids,
+                    self.ref_token_type_ids,
+                    self.ref_position_ids,
+                ),
+                return_convergence_delta=True,
+                additional_forward_args=(self.attention_mask),
+            )
+        elif self.position_ids is not None:
+            self._attributions, self.delta = self.lig.attribute(
+                inputs=(self.input_ids, self.position_ids),
+                baselines=(
+                    self.ref_input_ids,
+                    self.ref_position_ids,
+                ),
+                return_convergence_delta=True,
+                additional_forward_args=(self.attention_mask),
+            )
+        else:
+
+            self._attributions, self.delta = self.lig.attribute(
+                inputs=self.input_ids,
+                baselines=self.ref_input_ids,
+                return_convergence_delta=True,
+            )
 
     @property
     def word_attributions(self):
